@@ -33,7 +33,6 @@
 #include "ape_tag.h"
 #include "et_core.h"
 #include "log.h"
-#include "log.h"
 #include "misc.h"
 #include "setting.h"
 #include "charset.h"
@@ -72,7 +71,7 @@ gboolean Ape_Tag_Read_File_Tag (gchar *filename, File_Tag *FileTag)
     if ((file = fopen(filename, "rb")) == NULL)
     {
         gchar *filename_utf8 = filename_to_display(filename);
-        Log_Print(_("ERROR while opening file: '%s' (%s).\a"),filename_utf8, g_strerror(errno));
+        Log_Print(LOG_ERROR,_("ERROR while opening file: '%s' (%s)."),filename_utf8, g_strerror(errno));
         g_free(filename_utf8);
         return FALSE;
     }
