@@ -118,7 +118,7 @@ typedef enum
     FLAC_FILE,       // FLAC (lossless)           : .flac .fla
     MPC_FILE,        // MusePack                  : .mpc .mp+ .mpp
     MAC_FILE,        // Monkey's Audio (lossless) : .ape            (.mac)
-    //SPEEX_FILE,      // Speech audio files        : .spx
+    SPEEX_FILE,      // Speech audio files        : .spx
     OFR_FILE,        // OptimFROG (lossless)      : .ofr .ofs
     WAVPACK_FILE,    // Wavpack (lossless)        : .wv
     UNKNOWN_FILE
@@ -164,7 +164,7 @@ struct _Picture
     gchar   *description;
     gint     width;         /* Original width of the picture */
     gint     height;        /* Original height of the picture */
-    gulong   size;          /* Picture size in bits */
+    gulong   size;          /* Picture size in bytes (like stat) */
     guchar  *data;
     Picture *next;
 };
@@ -215,7 +215,7 @@ struct _ET_File_Info
     gint size;                  /* The size of file (in bytes) */
     gint duration;              /* The duration of file (in seconds) */
     gchar *mpc_profile;         /* MPC data */
-    gchar *mpc_version;         /* MPC data : encoder version */
+    gchar *mpc_version;         /* MPC data : encoder version  (also for Speex) */
 };
 
 
@@ -242,6 +242,10 @@ static const ET_File_Description ETFileDescription[] =
 #endif
 #ifdef ENABLE_OGG
     {OGG_FILE,     ".ogg",  OGG_TAG},
+    {OGG_FILE,     ".oga",  OGG_TAG},
+#endif
+#ifdef ENABLE_SPEEX
+    {SPEEX_FILE,   ".spx",  OGG_TAG},  // Implemented by Pierre Dumuid
 #endif
 #ifdef ENABLE_FLAC
     {FLAC_FILE,    ".flac", FLAC_TAG},
@@ -254,16 +258,14 @@ static const ET_File_Description ETFileDescription[] =
     {MAC_FILE,     ".mac",  APE_TAG},  // Implemented by Artur Polaczynski
     {OFR_FILE,     ".ofr",  APE_TAG},
     {OFR_FILE,     ".ofs",  APE_TAG},
-    //{WV_FILE,      ".wv",   APE_TAG},
 #ifdef ENABLE_MP4
     {MP4_FILE,     ".mp4",  MP4_TAG},  // Implemented by Michael Ihde
     {MP4_FILE,     ".m4a",  MP4_TAG},  // Implemented by Michael Ihde
     {MP4_FILE,     ".m4p",  MP4_TAG},  // Implemented by Michael Ihde
 #endif
 #ifdef ENABLE_WAVPACK
-    {WAVPACK_FILE, ".wv", WAVPACK_TAG}, // Implemented by Maarten Maathuis
+    {WAVPACK_FILE, ".wv",   WAVPACK_TAG}, // Implemented by Maarten Maathuis
 #endif
-//    {SPEEX_FILE,   ".spx",  OGG_TAG},  // Need to add some particularities...
     {UNKNOWN_FILE, "",      UNKNOWN_TAG}    /* This item must be placed to the end! */
 };
 
@@ -281,6 +283,8 @@ struct _ET_File
 
     guint ETFileKey;          /* Primary key to identify each item of the list (no longer used?) */
 
+    time_t FileModificationTime;            /* Save modification time of the file */
+
     ET_File_Description *ETFileDescription;
     gchar               *ETFileExtension;   /* Real extension of the file (keeping the case) (should be placed in ETFileDescription?) */
     ET_File_Info        *ETFileInfo;        /* Header infos: bitrate, duration, ... */
@@ -365,16 +369,18 @@ gboolean ET_Check_If_All_Files_Are_Saved (void);
 gboolean ET_Free_History_File_List       (void);
 gboolean ET_Free_Displayed_File_List     (void);
 
-ET_File   *ET_File_Item_New       (void);
-File_Name *ET_File_Name_Item_New  (void);
-File_Tag  *ET_File_Tag_Item_New   (void);
-gboolean   ET_Free_File_Tag_Item  (File_Tag *FileTag);
-gboolean   ET_Free_File_List_Item (ET_File *ETFile);
+ET_File      *ET_File_Item_New       (void);
+File_Name    *ET_File_Name_Item_New  (void);
+File_Tag     *ET_File_Tag_Item_New   (void);
+ET_File_Info *ET_File_Info_Item_New  (void);
+gboolean      ET_Free_File_Tag_Item  (File_Tag *FileTag);
+gboolean      ET_Free_File_List_Item (ET_File *ETFile);
 
-gboolean ET_Copy_File_Tag_Item         (ET_File *ETFile, File_Tag *FileTag);
-gboolean ET_Set_Field_File_Name_Item   (gchar **FileNameField, gchar *value);
-gboolean ET_Set_Field_File_Tag_Item    (gchar **FileTagField,  gchar *value);
-gboolean ET_Set_Field_File_Tag_Picture (gchar **FileTagField, Picture *pic);
+gboolean ET_Copy_File_Tag_Item          (ET_File *ETFile, File_Tag *FileTag);
+gboolean ET_Set_Field_File_Name_Item    (gchar **FileNameField, gchar *value);
+gboolean ET_Set_Filename_File_Name_Item (File_Name *FileName, gchar *filename_utf8, gchar *filename);
+gboolean ET_Set_Field_File_Tag_Item     (gchar **FileTagField,  gchar *value);
+gboolean ET_Set_Field_File_Tag_Picture  (Picture **FileTagField, Picture *pic);
 
 GList   *ET_Displayed_File_List_First       (void);
 GList   *ET_Displayed_File_List_Previous    (void);
@@ -408,7 +414,6 @@ gboolean ET_File_Name_Convert_Character          (gchar *filename_utf8);
 gchar   *ET_File_Name_Format_Extension           (ET_File *ETFile);
 gchar   *ET_File_Name_Generate                   (ET_File *ETFile, gchar *new_file_name);
 guint    ET_Get_Number_Of_Files_In_Directory     (gchar *path_utf8);
-gchar   *ET_Utf8_Validate_Full_String            (gchar *string_to_validate);
 
 gboolean ET_Detect_Changes_Of_File_Tag          (File_Tag  *FileTag1,  File_Tag  *FileTag2);
 
